@@ -13,8 +13,6 @@ use JSON;
 use Getopt::Std;
 use IO::String;
 
-our $VERSION = '1.9.7';
-
 has structFile            => ( isa => 'Str', is => 'ro', required => 1 );
 has confTreeFile          => ( isa => 'Str', is => 'ro', required => 1 );
 has managerConstantsFile  => ( isa => 'Str', is => 'ro', required => 1 );
@@ -22,6 +20,7 @@ has managerAttributesFile => ( isa => 'Str', is => 'ro', required => 1 );
 has defaultValuesFile     => ( isa => 'Str', is => 'ro', required => 1 );
 has confConstantsFile     => ( isa => 'Str', is => 'ro', required => 1 );
 has firstLmConfFile       => ( isa => 'Str', is => 'ro', required => 1 );
+has reverseTreeFile       => ( isa => 'Str', is => 'ro', required => 1 );
 
 my @managerAttrKeys = qw(keyTest keyMsgFail select type test msgFail default);
 my $format          = 'Creating %-69s: ';
@@ -37,6 +36,10 @@ my $mainTree;
 my @sessionTypes;
 my @simpleHashKeys;
 my @doubleHashKeys;
+my $authParameters;
+my $issuerParameters;
+my $samlServiceParameters;
+my $oidcServiceParameters;
 my $defaultValues;
 
 my $attributes = Lemonldap::NG::Manager::Build::Attributes::attributes();
@@ -150,16 +153,15 @@ sub run {
     $tmp = undef;
 
     printf STDERR $format, $self->managerConstantsFile;
-    my $sessionTypes = join( "', '", @sessionTypes );
 
     open F, ">", $self->managerConstantsFile or die($!);
     my $exportedVars = '$'
       . join( 'Keys $',
-        'simpleHash', 'specialNode', 'doubleHash', sort keys %cnodesRe )
-      . 'Keys $specialNodeHash @sessionTypes';
+        'simpleHash', 'doubleHash', 'specialNode', sort keys %cnodesRe )
+      . 'Keys $specialNodeHash $authParameters $issuerParameters $samlServiceParameters $oidcServiceParameters';
     print F <<EOF;
 # This file is generated by $module. Don't modify it by hand
-package Lemonldap::NG::Manager::Constants;
+package Lemonldap::NG::Common::Conf::ReConstants;
 
 use strict;
 use Exporter 'import';
@@ -179,8 +181,6 @@ our \$specialNodeHash = {
     oidcRPMetaDataNodes  => [qw(oidcRPMetaDataOptions oidcRPMetaDataExportedVars oidcRPMetaDataOptionsExtraClaims)],
 };
 
-our \@sessionTypes = ( '$sessionTypes' );
-
 EOF
 
     # Reinitialize $attributes
@@ -200,6 +200,22 @@ EOF
     foreach ( sort keys %cnodesRe ) {
         print F "our \$${_}Keys = '$cnodesRe{$_}';\n";
     }
+    print F "\n";
+
+    foreach (qw(authParameters issuerParameters)) {
+        $tmp = "our \$$_ = {\n";
+        no strict 'refs';
+        foreach my $k ( sort keys %$$_ ) {
+            my $v = $$_->{$k};
+            $tmp .= "  $k => [qw(" . join( ' ', @$v ) . ")],\n";
+        }
+        print F "$tmp};\n";
+    }
+    foreach (qw(samlServiceParameters oidcServiceParameters)) {
+        no strict 'refs';
+        $tmp = "our \$$_ = [qw(" . join( ' ', @$$_ ) . ")];\n";
+        print F "$tmp";
+    }
 
     print F "\n1;\n";
     close F;
@@ -236,7 +252,7 @@ $defaultAttr}
 
     printf STDERR $format, $self->confConstantsFile;
     $ra = Regexp::Assemble->new;
-    foreach ( @simpleHashKeys, @doubleHashKeys, sort keys %cnodesRe ) {
+    foreach ( @simpleHashKeys, sort keys %cnodesRe ) {
         $ra->add($_);
     }
     foreach (
@@ -252,6 +268,7 @@ $defaultAttr}
         $ra->add($_);
     }
 
+    my $sessionTypes = join( "', '", @sessionTypes );
     my $confConstants =
       "our \$hashParameters = qr/^" . $ra->as_string . "\$/;\n";
     open( F, ">", $self->confConstantsFile ) or die($!);
@@ -282,6 +299,9 @@ use constant MANAGERSECTION  => "manager";
 use constant SESSIONSEXPLORERSECTION => "sessionsExplorer";
 use constant APPLYSECTION            => "apply";
 $confConstants
+our \@sessionTypes = ( '$sessionTypes' );
+
+sub NO {qr/^(?:off|no|0)?\$/i}
 
 our %EXPORT_TAGS = (
     'all' => [
@@ -300,7 +320,9 @@ our %EXPORT_TAGS = (
           MANAGERSECTION
           SESSIONSEXPLORERSECTION
           APPLYSECTION
+          NO
           \$hashParameters
+          \@sessionTypes 
           )
     ]
 );
@@ -350,6 +372,18 @@ $managerAttr}
     print STDERR "done\n";
 
     $self->buildZeroConf();
+
+    printf STDERR $format, $self->reverseTreeFile;
+    open( F, ">", $self->reverseTreeFile ) or die($!);
+    $jsonEnc->pretty(0);
+    print F $jsonEnc->encode(
+        $self->reverseScan(
+            Lemonldap::NG::Manager::Build::Tree::tree(),
+            '', {}
+        )
+    );
+    close F;
+    print STDERR "done\n";
 }
 
 sub buildZeroConf {
@@ -404,11 +438,34 @@ sub scanTree {
         elsif ( ref($leaf) ) {
             $jleaf->{title} = $jleaf->{id} = $leaf->{title};
             $jleaf->{type} = $leaf->{form} if ( $leaf->{form} );
+            if ( $leaf->{title} =~ /^((?:oidc|saml)Service)MetaData$/ ) {
+                no strict 'refs';
+                my @tmp = $self->scanLeaf( $leaf->{nodes} );
+                ${ $1 . 'Parameters' } = \@tmp;
+            }
             foreach my $n (qw(nodes nodes_cond)) {
                 if ( $leaf->{$n} ) {
                     $jleaf->{"_$n"} = [];
                     $self->scanTree( $leaf->{$n}, $jleaf->{"_$n"}, $prefix,
                         "$path.$nodeName\[$ord\]" );
+                    if (
+                        (
+                                $leaf->{title} eq 'authParams'
+                            and $n eq 'nodes_cond'
+                        )
+                        or $leaf->{title} eq 'issuerParams'
+                      )
+                    {
+                        my $vn = $leaf->{title};
+                        $vn =~ s/Params$/Parameters/;
+                        foreach my $sn ( @{ $leaf->{$n} } ) {
+                            no strict 'refs';
+                            my @cn = $self->scanLeaf( $sn->{nodes} );
+                            ${$vn}->{ $sn->{title} } = \@cn;
+                        }
+                    }
+                    elsif ( $leaf->{title} eq 'issuerParams' ) {
+                    }
                     if ( $n eq 'nodes_cond' ) {
                         foreach my $sn ( @{ $jleaf->{"_$n"} } ) {
                             $sn->{show} = 'false';
@@ -452,7 +509,9 @@ sub scanTree {
                 $jleaf->{$w} = $attr->{$w} if ( defined $attr->{$w} );
             }
             if ( defined $jleaf->{default} ) {
-                $defaultValues->{$leaf} = $jleaf->{default};
+                unless ( $attr->{type} eq 'bool' and $jleaf->{default} == 0 ) {
+                    $defaultValues->{$leaf} = $jleaf->{default};
+                }
                 if ( ref( $jleaf->{default} ) ) {
                     $jleaf->{default} = [];
                     my $type = $attr->{type};
@@ -495,6 +554,53 @@ sub scanTree {
     }
 }
 
+sub scanLeaf {
+    my ( $self, $tree ) = @_;
+    my @res;
+    foreach my $k (@$tree) {
+        if ( ref $k ) {
+            push @res, $self->scanLeaf1( $k->{nodes} || $k->{group} );
+        }
+        else {
+            push @res, $k;
+        }
+    }
+    return @res;
+}
+
+sub scanLeaf1 {
+    my ( $self, $tree ) = @_;
+    my @res;
+    foreach my $k (@$tree) {
+        if ( ref $k ) {
+            push @res, $self->scanLeaf( $k->{nodes} || $k->{group} );
+        }
+        else {
+            push @res, $k;
+        }
+    }
+    return @res;
+}
+
+sub reverseScan {
+    my ( $self, $tree, $path, $res ) = @_;
+    foreach my $elem (@$tree) {
+        $elem =~ s/^\*//;
+        if ( ref($elem) eq 'HASH' ) {
+            foreach (qw(nodes nodes_cond group)) {
+                $self->reverseScan( $elem->{$_}, "$path$elem->{title}/", $res )
+                  if ( $elem->{$_} );
+            }
+        }
+        else {
+            my $tmp = $path;
+            $tmp =~ s#/$##;
+            $res->{$elem} = $tmp;
+        }
+    }
+    return $res;
+}
+
 1;
 __END__
 
@@ -510,12 +616,13 @@ system.
   use Lemonldap::NG::Manager::Build;
   
   Lemonldap::NG::Manager::Build->run(
-    structFile            => "site/static/struct.json",
-    confTreeFile          => "site/static/js/conftree.js",
-    managerConstantsFile  => "lib/Lemonldap/NG/Manager/Constants.pm",
+    structFile            => "site/htdocs/static/struct.json",
+    confTreeFile          => "site/htdocs/static/js/conftree.js",
+    managerConstantsFile  => "lib/Lemonldap/NG/Common/Conf/ReConstants.pm",
     managerAttributesFile => 'lib/Lemonldap/NG/Manager/Attributes.pm',
     defaultValuesFile     => "lib/Lemonldap/NG/Common/Conf/DefaultValues.pm",
-    firstLmConfFile       => "_example/conf/lmConf-1.js",
+    firstLmConfFile       => "_example/conf/lmConf-1.json",
+    reverseTreeFile       => "site/htdocs/static/reverseTree.json",
   );
 
 =head1 DESCRIPTION
@@ -541,6 +648,43 @@ root of the Lemonldap::NG project sources).
 
 See below for details.
 
+=head3 Files generated
+
+The `scripts/jsongenerator.pl` file uses Lemonldap::NG::Manager::Build::Attributes,
+Lemonldap::NG::Manager::Build::Tree and Lemonldap::NG::Manager::Build::CTrees to generate
+
+=over
+
+=item `site/htdocs/static/struct.json`:
+
+the main file that contains the tree view;
+
+=item `site/htdocs/static/js/conftree.js`:
+
+generates sub tree for virtualhosts and SAML and OpenID-Connect partners;
+
+=item `site/htdocs/static/reverseTree.json`:
+
+map used by manager diff to find attribute position in the tree;
+
+=item `Lemonldap::NG::Manager::Constants`:
+
+constants used by all Perl manager components;
+
+=item `Lemonldap::NG::Common::Conf::DefaultValues`:
+
+constants used to read configuration;
+
+=item `Lemonldap::NG::Manager::Attributes`:
+
+parameters attributes used by the manager during configuration upload;
+
+=item lmConf-1.json:
+
+first configuration in file format;
+
+=back
+
 =head3 Attribute declaration
 
 Set your new attribute as a key of attributes() function that points to a hash
@@ -652,36 +796,33 @@ to be displayed in the same form
 =item nodes_cond:
 
 array of sub nodes that will be displayed with a filter. Not
-yet documented here, see the source code of site/static/js/filterFunctions.js.
+yet documented here, see the source code of site/htdocs/static/js/filterFunctions.js.
 
 =item nodes_filter:
 
-filter entry in site/static/js/filterFunctions.js for the same feature.
+filter entry in site/htdocs/static/js/filterFunctions.js for the same feature.
 
 =back
 
 =head1 SEE ALSO
 
-L<Lemonldap::NG::Manager>, L<http://lemonldap-ng.org/>
+L<http://lemonldap-ng.org/>
 
 =head1 AUTHORS
 
 =over
 
-=item Clement Oudot, E<lt>clem.oudot@gmail.comE<gt>
-
-=item François-Xavier Deltombe, E<lt>fxdeltombe@gmail.com.E<gt>
-
-=item Xavier Guimard, E<lt>x.guimard@free.frE<gt>
-
-=item Thomas Chemineau, E<lt>thomas.chemineau@gmail.comE<gt>
+=item LemonLDAP::NG team L<http://lemonldap-ng.org/team>
 
 =back
 
 =head1 BUG REPORT
 
 Use OW2 system to report bug or ask for features:
-L<https://gitlab.ow2.org/lemonldap-ng/lemonldap-ng/issues>
+L<http://jira.ow2.org>
+
+Note that if you want to post a ticket for a conf upload problem, please
+see L<Lemonldap::NG::Manager::Conf::Parser> before.
 
 =head1 DOWNLOAD
 
@@ -690,13 +831,7 @@ L<http://forge.objectweb.org/project/showfiles.php?group_id=274>
 
 =head1 COPYRIGHT AND LICENSE
 
-=over
-
-=item Copyright (C) 2015-2016 by Xavier Guimard, E<lt>x.guimard@free.frE<gt>
-
-=item Copyright (C) 2015-2016 by Clément Oudot, E<lt>clem.oudot@gmail.comE<gt>
-
-=back
+See COPYING file for details.
 
 This library is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
